const {Router} = require('express');
const Redis = require('ioredis');
const fs = require('fs');
const moment = require('moment');
const request = require('request');

const {readFile} = require('fs').promises

const port = process.env.PORT || 6379;
const REDIS_EXPIRY_SECONDS = 600;
const DATE_FORMAT = "YYYY-MM-DD HH:mm:ss";
const URL_DBROBO = "http://dbrobo.mgul.ac.ru/core/deb.php?fdate=";
const SDATE_QUERY_PARAM = '&sdate=';

var router = Router();

const redis = new Redis({
    port: port,
});

router.get('/', (req, res) => {
    res.send('respond with a resource');
});

async function readAndParseCalibrationFile(file) {
    try {
        const data = await readFile(file, 'utf-8');
        return JSON.parse(data);
    } catch (error) {
        throw error;
    }
}

function getCalibrationNameAndSerial(calibrationData) {
    try {
        const calibrationKeys = Object.keys(calibrationData);
        let deviceInfoArray = [];
        for (const key in calibrationKeys) {
            const deviceName = calibrationData[key].uName;
            const deviceSerial = calibrationData[key].serial;
            deviceInfoArray.push([deviceName, deviceSerial]);
        }
        return deviceInfoArray;
    } catch (error) {
        throw error;
    }
}

function getCalibrationSensorNames(calibrationData) {
    let responseArray = [];

    for (const calibrationEntry of calibrationData) {
        let sensorNamesArray = [];
        const sensors = calibrationEntry.sensors;

        for (const sensor of sensors) {
            sensorNamesArray.push(sensor.sensor);
        }
        responseArray.push(sensorNamesArray);
    }
    return responseArray;
}

async function getAllRedisKeys() {
    return redis.keys('*');
}

function getNecessaryKeys(startDate, endDate, keysArray) {
    const startTimestamp = moment(startDate, DATE_FORMAT).valueOf();
    const endTimestamp = moment(endDate, DATE_FORMAT).valueOf();

    return keysArray.filter(function (key) {
        let keyTimestamp = moment(key, DATE_FORMAT).valueOf();
        return keyTimestamp >= startTimestamp && keyTimestamp <= endTimestamp;
    })
}

function checkDateRangeInRedis(formattedStartDate, formattedEndDate, keysArray) {
    const startDate = keysArray[0];
    const endDate = keysArray[keysArray.length - 1];

    let startDifference = moment(endDate).diff(moment(formattedStartDate), 'minutes');
    if (startDate <= formattedStartDate && formattedStartDate < endDate && Math.abs(startDifference) >= 5) {
        return true;
    }
    let endDifference = moment(formattedEndDate).diff(moment(startDate), 'minutes');
    return formattedStartDate <= startDate && startDate < formattedEndDate && Math.abs(endDifference) >= 5;
}

function sortRedisKeys(keysArray) {
    return keysArray.sort(function (a, b) {
        return a > b ? 1 : a < b ? -1 : 0;
    })
}

async function sortedAndFilteredKeys(formattedStartDate, formattedEndDate) {
    return getNecessaryKeys(formattedStartDate, formattedEndDate, sortRedisKeys(await getAllRedisKeys()));
}

async function updateDataInRedis(url) {
    try {
        const databaseResponse = await downloadPage(url);
        const keys = Object.keys(databaseResponse);

        const existsPipeline = redis.pipeline();
        const updatePipeline = redis.pipeline();

        for (const key of keys) {
            const currentDate = databaseResponse[key].Date;
            existsPipeline.exists(currentDate);
        }

        const existsResults = await existsPipeline.exec();
        for (let i = 0; i < keys.length; i++) {
            const key = keys[i];
            const currentDate = databaseResponse[key].Date;
            const keyExistsInRedis = existsResults[i][1] === 1;

            if (!keyExistsInRedis) {
                const newDataArray = JSON.stringify(databaseResponse[key]);
                updatePipeline.set(currentDate, newDataArray, 'EX', REDIS_EXPIRY_SECONDS);
            }
        }

        await updatePipeline.exec();
    } catch (error) {
        throw error;
    }
}

function processAndAggregateLastMeasurements(respondParsed, keysArray) {
    let aggregated = {};
    for (let j = 0; j < respondParsed.length; j++) {
        let responseItem = JSON.parse(respondParsed[j]);

        let key = responseItem.uName + "@";
        if (responseItem.serial !== undefined) {
            key += responseItem.serial;
        }
        if (!aggregated.hasOwnProperty(key) || aggregated[key].Date < responseItem.Date) {
            if (keysArray.includes(key))
                aggregated[key] = responseItem;
        }
    }
    return aggregated;
}

function processAndAggregateResponsesList(respondParsed, keysArray) {
    let aggregated = {};
    for (let j = 0; j < respondParsed.length; j++) {
        let responseItem = JSON.parse(respondParsed[j]);

        let key = responseItem.uName + "@";
        if (responseItem.serial !== undefined) {
            key += responseItem.serial;
        }
        if (keysArray.includes(key))
            aggregated[j] = responseItem;
    }
    return aggregated;
}

function downloadPage(url) {
    return new Promise((resolve, reject) => {
        request(url, (error, response, body) => {
            if (error) {
                reject(error);
                return;
            }
            if (response.statusCode !== 200) {
                reject(response.statusCode);
                return;
            }
            let parsedBody;
            try {
                parsedBody = JSON.parse(body);
            } catch (jsonError) {
                reject(jsonError.message);
            }
            resolve(parsedBody);
        });
    });
}

async function requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys) {
    try {
        let flag = false;
        const secondsThreshold = 360;
        const firstKey = filteredKeys[0];
        const formattedFirstKey = moment(firstKey).format(DATE_FORMAT);
        const startDifference = moment(formattedFirstKey).diff(moment(formattedStartDate), 'seconds');
        if (Math.abs(startDifference) > secondsThreshold) {
            flag = true;
            const dataUrl = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedFirstKey}`;
            await updateDataInRedis(dataUrl);
        }
        for (let i = 0; i < filteredKeys.length - 2; i++) {
            const firstDate = filteredKeys[i];
            const secondDate = filteredKeys[i + 1];

            const formattedFirstDate = moment(firstDate).format(DATE_FORMAT);
            const formattedSecondDate = moment(secondDate).format(DATE_FORMAT);

            const secondsDifference = moment(formattedSecondDate).diff(moment(formattedFirstDate), 'seconds');
            if (secondsDifference > secondsThreshold) {
                flag = true;
                const dataUrl = `${URL_DBROBO}${formattedFirstDate}${SDATE_QUERY_PARAM}${formattedSecondDate}`;
                await updateDataInRedis(dataUrl);
            }
        }
        const lastKey = filteredKeys[filteredKeys.length - 1];
        const formattedLastKey = moment(lastKey).format(DATE_FORMAT);
        const endDifference = moment(formattedEndDate).diff(moment(formattedLastKey), 'seconds');
        if (Math.abs(endDifference) > secondsThreshold) {
            flag = true;
            const dataUrl = `${URL_DBROBO}${formattedLastKey}${SDATE_QUERY_PARAM}${formattedEndDate}`;
            await updateDataInRedis(dataUrl);
        }
        return flag ? await sortedAndFilteredKeys(formattedStartDate, formattedEndDate) : filteredKeys;
    } catch (error) {
        throw error;
    }
}

async function respondToDataQuery(keys) {
    try {
        const pipeline = redis.pipeline();
        let responseArray = [];

        for (const key of keys) {
            pipeline.get(key);
        }
        const result = await pipeline.exec();

        for (const value of result) {
            responseArray.push(value[1]);
        }

        return responseArray;
    } catch (error) {
        throw error;
    }
}

async function respondToDataQueryParsed(keys) {
    try {
        const pipeline = redis.pipeline();
        let responseArray = [];

        for (const key of keys) {
            pipeline.get(key);
        }
        const result = await pipeline.exec();

        for (const value of result) {
            responseArray.push(JSON.parse(value[1]));
        }

        return responseArray;
    } catch (error) {
        throw error;
    }
}

function applyCalibrationsToData(calibrationData, filteredResponse) {
    const calibrationKeys = Object.keys(calibrationData);
    const availableDevices = getCalibrationNameAndSerial(calibrationData);
    const sensorNames = getCalibrationSensorNames(calibrationData, calibrationKeys);

    let temporaryResponse = filteredResponse;
    let responseKey = Object.keys(temporaryResponse);

    for (let i = 0; i < responseKey.length; i++) {
        const currentResponse = temporaryResponse[responseKey[i]];
        const uNameToSearch = currentResponse.uName;
        const serialToSearch = currentResponse.serial;

        const index = availableDevices.findIndex(device => device[0] === uNameToSearch && device[1] === serialToSearch);
        if (index === -1) {
            continue;
        }

        const currentResponseData = temporaryResponse[responseKey[i]].data;
        const measurementDatetime = temporaryResponse[responseKey[i]].Date;
        const sensorNamesArray = sensorNames[index];

        for (let k = 0; k < sensorNamesArray.length; k++) {
            const sensorCalibrations = calibrationData[index].sensors[k].calibr;
            let currentSensorName = sensorNamesArray[k];
            let sensorIndex = Object.keys(currentResponseData).indexOf(currentSensorName);

            if (sensorIndex === -1) {
                continue;
            }

            let calibrationDatetime;
            let calibrationDate = currentSensorName + '_calibr_date';
            let calibrationDateArray = [];

            for (let x = 0; x < sensorCalibrations.length; x++) {
                calibrationDatetime = JSON.stringify(sensorCalibrations[x].datetime);
                calibrationDateArray.push(calibrationDatetime);
            }

            calibrationDateArray.push(measurementDatetime);
            calibrationDateArray = calibrationDateArray.sort((a, b) => b.date - a.date);
            const measurementDatetimeIndex = calibrationDateArray.indexOf(measurementDatetime);
            let xIndex;

            for (let x = 0; x < sensorCalibrations.length; x++) {
                calibrationDatetime = JSON.stringify(sensorCalibrations[x].datetime);
                if (calibrationDatetime === calibrationDateArray[measurementDatetimeIndex - 1]) xIndex = x;
            }

            if (moment(measurementDatetime).isAfter(calibrationDatetime) === false) {
                const calibrationN = JSON.stringify(sensorCalibrations[xIndex].data.n);
                const calibrationAI = sensorCalibrations[xIndex].data.ai;

                if (calibrationN === undefined || calibrationAI === undefined || calibrationN < 1) {
                    continue;
                }

                const responseEntries = Object.entries(currentResponseData);
                if (responseEntries[sensorIndex] !== undefined) {
                    let default_value = responseEntries[sensorIndex][1];
                    responseEntries[sensorIndex][1] = 0;
                    for (let u = calibrationAI.length - 1; u > 0; u--) {
                        responseEntries[sensorIndex][1] = (responseEntries[sensorIndex][1] + calibrationAI[u]) * default_value;
                    }
                    responseEntries[sensorIndex][1] = responseEntries[sensorIndex][1] + calibrationAI[0];
                }

                responseEntries[sensorIndex][1] = responseEntries[sensorIndex][1].toString();
                responseEntries.push([calibrationDate, sensorCalibrations[xIndex].datetime])

                let temporaryObj = {};
                for (let y = 0; y < responseEntries.length; y++) {
                    let name = responseEntries[y][0];
                    temporaryObj[name] = responseEntries[y][1];
                }
                temporaryResponse[responseKey[i]].data = temporaryObj;

            }
        }

    }
    return temporaryResponse;
}

setInterval( function () {
    request({
        url: "http://dbrobo.mgul.ac.ru/core/deb.php?calibration",
        method: "GET", //Запрос типа GET
        timeout: 60000,
        followRedirect: true,
        maxRedirects: 10
    }, function (error, response, body) {
        if (!error && response.statusCode == 200) {
            console.log('Calibration Data has been loaded');
            fs.writeFile("CalibrationData.txt", response.body.toString(), function (error) {
                if (error) throw error;
            });

        } else {
            console.log('error' + response.statusCode);
        }
    });
}, 600000);

moment.suppressDeprecationWarnings = true;

redis.flushdb(function (err, succeeded) {
    console.log(succeeded);
});

// Для получения НЕОТКАЛИБРОВАННЫХ ИЗМЕРЕНИЙ в указанный временной промежуток.
// Необходимо указать начальную дату и время (:fdate), конечную дату и время (:sdate).
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/not_calibr/log/2023-01-01%2000:00:00/2023-01-01%2000:08:00
router.get('/not_calibr/log/:fdate/:sdate', async function (req, res) {
    try {
        const startDateParam = req.params.fdate;
        const endDateParam = req.params.sdate;
        const formattedStartDate = moment(startDateParam).format(DATE_FORMAT);
        const formattedEndDate = moment(endDateParam).format(DATE_FORMAT);

        let duration = moment.duration(moment(formattedEndDate).diff(moment(formattedStartDate))).asDays();

        if (duration > 7) {
            res.send([]);
        } else {
            const sortedKeys = sortRedisKeys(await getAllRedisKeys());
            let filteredKeys;

            if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
                filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
                filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
            } else {
                const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
                await updateDataInRedis(dateRangeQueryString);
                filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
            }
            filteredKeys = filteredKeys.reverse();

            const responseArray = await respondToDataQueryParsed(filteredKeys);
            const finalResponse = responseArray.reduce((acc, val) => acc.concat(val), []);
            const answer = {...finalResponse};

            if (Object.keys(answer).length === 0) {
                res.send([]);
            }

            res.send(answer);
        }
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});

// Для получения НЕОТКАЛИБРОВАННЫХ ИЗМЕРЕНИЙ в указанный временной промежуток для определенного списка устройств.
// Необходимо указать начальную дату и время (:fdate), конечную дату и время (:sdate), а также список устройств в параметре "list".
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/not_calibr/log/2023-01-01%2000:00:00/2023-01-01%2000:08:00/Опорный%20барометр@02&Паскаль@01&Паскаль@02
router.get('/not_calibr/log/:fdate/:sdate/:list', async function (req, res) {
    try {
        let selectedKeys = req.params.list.split('&');

        const startDateParam = req.params.fdate;
        const endDateParam = req.params.sdate;
        const formattedStartDate = moment(startDateParam).format(DATE_FORMAT);
        const formattedEndDate = moment(endDateParam).format(DATE_FORMAT);

        let duration = moment.duration(moment(formattedEndDate).diff(moment(formattedStartDate))).asDays();
        if (duration > 7) {
            res.send([]);
        } else {
            let sortedKeys = sortRedisKeys(await getAllRedisKeys());
            let filteredKeys;

            if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
                filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
                filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
            } else {
                const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
                await updateDataInRedis(dateRangeQueryString);
                filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
            }
            filteredKeys = filteredKeys.reverse();

            const responseArray = await respondToDataQuery(filteredKeys);
            const parsedResponse = JSON.parse(JSON.stringify(responseArray).trim());
            const aggregatedResult = processAndAggregateResponsesList(parsedResponse, selectedKeys);
            const answer = Object.values(aggregatedResult);

            if (Object.keys(answer).length === 0) {
                res.send([]);
            }

            res.send(answer);
        }
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});

// Для получения НЕОТКАЛИБРОВАННЫХ ДАННЫХ ПО ПОСЛЕДНИМ ИЗМЕРЕНИЯМ (за последние 10 минут) с определенного списка устройств.
// Необходимо указать список устройств в параметре "list".
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/not_calibr/last_measurement_list/Опорный%20барометр@02&Паскаль@01&Паскаль@02
router.get('/not_calibr/last_measurement_list/:list', async function (req, res) {
    try {
        let selectedKeys = req.params.list.split('&');

        const now = moment();

        const formattedStartDate = moment(now).subtract(10, 'minutes').format(DATE_FORMAT);
        const formattedEndDate = moment(now).format(DATE_FORMAT);

        let sortedKeys = sortRedisKeys(await getAllRedisKeys());
        let filteredKeys;

        if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
            filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
            filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
        } else {
            const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
            await updateDataInRedis(dateRangeQueryString);
            filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
        }
        filteredKeys = filteredKeys.reverse();

        const responseArray = await respondToDataQuery(filteredKeys);
        const parsedResponse = JSON.parse(JSON.stringify(responseArray).trim());
        const aggregatedResult = processAndAggregateLastMeasurements(parsedResponse, selectedKeys);
        const answer = Object.values(aggregatedResult);

        if (Object.keys(answer).length === 0) {
            res.send([]);
        }

        res.send(answer);
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});


// Для получения ОТКАЛИБРОВАННЫХ ИЗМЕРЕНИЙ в указанный временной промежуток.
// Необходимо указать начальную дату и время (:fdate), конечную дату и время (:sdate).
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/calibr/log/2023-01-01%2000:00:00/2023-01-01%2000:08:00
router.get('/calibr/log/:fdate/:sdate', async function (req, res) {
    try {
        const startDateParam = req.params.fdate;
        const endDateParam = req.params.sdate;
        const formattedStartDate = moment(startDateParam).format(DATE_FORMAT);
        const formattedEndDate = moment(endDateParam).format(DATE_FORMAT);

        let duration = moment.duration(moment(formattedEndDate).diff(moment(formattedStartDate))).asDays();
        if (duration > 7) {
            res.send([]);
        } else {
            let sortedKeys = sortRedisKeys(await getAllRedisKeys());
            let filteredKeys;

            if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
                filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
                filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
            } else {
                const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
                await updateDataInRedis(dateRangeQueryString);
                filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
            }
            filteredKeys = filteredKeys.reverse();

            const responseArray = await respondToDataQueryParsed(filteredKeys);
            const filteredResponse = responseArray.reduce((acc, val) => acc.concat(val), []);

            const calibrationData = await readAndParseCalibrationFile('CalibrationData.txt');
            const finalResponse = applyCalibrationsToData(calibrationData, filteredResponse);

            let answer = {...finalResponse};

            if (Object.keys(answer).length === 0) {
                res.send([]);
            }

            res.send(answer);
        }
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});

// Для получения ОТКАЛИБРОВАННЫХ ИЗМЕРЕНИЙ в указанный временной промежуток для определенного списка устройств.
// Необходимо указать начальную дату и время (:fdate), конечную дату и время (:sdate), а также список устройств в параметре "list".
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/calibr/log/2023-12-17%2011:00:00/2023-12-17%2014:07:00/Опорный%20барометр@02&Паскаль@01
router.get('/calibr/log/:fdate/:sdate/:list', async function (req, res) {
    try {
        let selectedKeys = req.params.list.split('&');

        const startDateParam = req.params.fdate;
        const endDateParam = req.params.sdate;
        const formattedStartDate = moment(startDateParam).format(DATE_FORMAT);
        const formattedEndDate = moment(endDateParam).format(DATE_FORMAT);

        let duration = moment.duration(moment(formattedEndDate).diff(moment(formattedStartDate))).asDays();
        if (duration > 7) {
            res.send([]);
        } else {
            let sortedKeys = sortRedisKeys(await getAllRedisKeys());
            let filteredKeys;

            if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
                filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
                filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
            } else {
                const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
                await updateDataInRedis(dateRangeQueryString);
                filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
            }
            filteredKeys = filteredKeys.reverse();

            const responseArray = await respondToDataQuery(filteredKeys);
            const parsedResponse = JSON.parse(JSON.stringify(responseArray).trim());
            const aggregatedResult = processAndAggregateResponsesList(parsedResponse, selectedKeys);
            const filteredResponse = Object.values(aggregatedResult);

            const calibrationData = await readAndParseCalibrationFile('CalibrationData.txt');
            const finalResponse = applyCalibrationsToData(calibrationData, filteredResponse);

            let answer = {...finalResponse};

            if (Object.keys(answer).length === 0) {
                res.send([]);
            }

            res.send(answer);
        }
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});

// Для получения ОТКАЛИБРОВАННЫХ ДАННЫХ ПО ПОСЛЕДНИМ ИЗМЕРЕНИЯМ (за последние 10 минут) с определенного списка устройств.
// Необходимо указать список устройств в параметре "list".
// Пример URL: http://webrobo.mgul.ac.ru:3000/db_api_REST/calibr/last_measurement_list/Опорный%20барометр@02&Паскаль@01&Паскаль@02
router.get('/calibr/last_measurement_list/:list', async function (req, res) {
    try {
        let selectedKeys = req.params.list.split('&');

        const now = moment();
        const formattedStartDate = moment(now).subtract(10, 'minutes').format(DATE_FORMAT);
        const formattedEndDate = moment(now).format(DATE_FORMAT);

        let sortedKeys = sortRedisKeys(await getAllRedisKeys());
        let filteredKeys;

        if (checkDateRangeInRedis(formattedStartDate, formattedEndDate, sortedKeys)) {
            filteredKeys = getNecessaryKeys(formattedStartDate, formattedEndDate, sortedKeys);
            filteredKeys = await requestAdditionalDataIfMissing(formattedStartDate, formattedEndDate, filteredKeys);
        } else {
            const dateRangeQueryString = `${URL_DBROBO}${formattedStartDate}${SDATE_QUERY_PARAM}${formattedEndDate}`;
            await updateDataInRedis(dateRangeQueryString);
            filteredKeys = await sortedAndFilteredKeys(formattedStartDate, formattedEndDate);
        }
        filteredKeys = filteredKeys.reverse();

        const responseArray = await respondToDataQuery(filteredKeys);
        const parsedResponse = JSON.parse(JSON.stringify(responseArray).trim());
        const aggregatedResult = processAndAggregateLastMeasurements(parsedResponse, selectedKeys);
        const filteredResponse = Object.values(aggregatedResult);

        const calibrationData = await readAndParseCalibrationFile('CalibrationData.txt');
        const finalResponse = applyCalibrationsToData(calibrationData, filteredResponse);

        let answer = {...finalResponse};

        if (Object.keys(answer).length === 0) {
            res.send([]);
        }

        res.send(answer);
    } catch (error) {
        console.error('Error:', error);
        res.status(500).send(error);
    }
});

module.exports = router;
